package sample.control;


public final class PhoneControllerContext
    extends FSMContext
{
//---------------------------------------------------------------
// Member methods.
//

    public PhoneControllerContext(PhoneController owner)
    {
        super();

        _owner = owner;
        setState(PhoneControllerFSM.Wait_for_Call);
        PhoneControllerFSM.Wait_for_Call.Entry(this);
    }

    public synchronized void answer()
    {
        _transition = "answer";
        getState().answer(this);
        _transition = "";
        return;
    }

    public synchronized void callFailure()
    {
        _transition = "callFailure";
        getState().callFailure(this);
        _transition = "";
        return;
    }

    public synchronized void callFinished()
    {
        _transition = "callFinished";
        getState().callFinished(this);
        _transition = "";
        return;
    }

    public synchronized void callSuccess()
    {
        _transition = "callSuccess";
        getState().callSuccess(this);
        _transition = "";
        return;
    }

    public synchronized void cancelCall()
    {
        _transition = "cancelCall";
        getState().cancelCall(this);
        _transition = "";
        return;
    }

    public synchronized void dial(String number)
    {
        _transition = "dial";
        getState().dial(this, number);
        _transition = "";
        return;
    }

    public synchronized void haveCall(String number)
    {
        _transition = "haveCall";
        getState().haveCall(this, number);
        _transition = "";
        return;
    }

    public synchronized void makeCall()
    {
        _transition = "makeCall";
        getState().makeCall(this);
        _transition = "";
        return;
    }

    public synchronized void noAnswer()
    {
        _transition = "noAnswer";
        getState().noAnswer(this);
        _transition = "";
        return;
    }

    public PhoneControllerState getState()
        throws StateUndefinedException
    {
        if (_state == null)
        {
            throw(
                new StateUndefinedException());
        }

        return ((PhoneControllerState) _state);
    }

    protected PhoneController getOwner()
    {
        return (_owner);
    }

//---------------------------------------------------------------
// Member data.
//

    transient private PhoneController _owner;

//---------------------------------------------------------------
// Inner classes.
//

    public static abstract class PhoneControllerState
        extends State
    {
    //-----------------------------------------------------------
    // Member methods.
    //

        protected PhoneControllerState(String name, int id)
        {
            super (name, id);
        }

        protected void Entry(PhoneControllerContext context) {}
        protected void Exit(PhoneControllerContext context) {}

        protected void answer(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void callFailure(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void callFinished(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void callSuccess(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void cancelCall(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void dial(PhoneControllerContext context, String number)
        {
            Default(context);
        }

        protected void haveCall(PhoneControllerContext context, String number)
        {
            Default(context);
        }

        protected void makeCall(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void noAnswer(PhoneControllerContext context)
        {
            Default(context);
        }

        protected void Default(PhoneControllerContext context)
        {
            throw (
                new TransitionUndefinedException(
                    "State: " +
                    context.getState().getName() +
                    ", Transition: " +
                    context.getTransition()));
        }

    //-----------------------------------------------------------
    // Member data.
    //
    }

    /* package */ static abstract class PhoneControllerFSM
    {
    //-----------------------------------------------------------
    // Member methods.
    //

    //-----------------------------------------------------------
    // Member data.
    //

        //-------------------------------------------------------
        // Statics.
        //
        public static PhoneControllerFSM_Default.PhoneControllerFSM_Wait_for_Call Wait_for_Call;
        public static PhoneControllerFSM_Default.PhoneControllerFSM_Dialing Dialing;
        public static PhoneControllerFSM_Default.PhoneControllerFSM_Calling Calling;
        public static PhoneControllerFSM_Default.PhoneControllerFSM_Answering Answering;
        public static PhoneControllerFSM_Default.PhoneControllerFSM_Waiting_for_Answer Waiting_for_Answer;
        private static PhoneControllerFSM_Default Default;

        static
        {
            Wait_for_Call = new PhoneControllerFSM_Default.PhoneControllerFSM_Wait_for_Call("PhoneControllerFSM.Waiting_for_Call", 20);
            Dialing = new PhoneControllerFSM_Default.PhoneControllerFSM_Dialing("PhoneControllerFSM.Dialing", 21);
            Calling = new PhoneControllerFSM_Default.PhoneControllerFSM_Calling("PhoneControllerFSM.Calling", 22);
            Answering = new PhoneControllerFSM_Default.PhoneControllerFSM_Answering("PhoneControllerFSM.Answering", 23);
            Waiting_for_Answer = new PhoneControllerFSM_Default.PhoneControllerFSM_Waiting_for_Answer("PhoneControllerFSM.Waiting_for_Answer", 24);
            Default = new PhoneControllerFSM_Default("PhoneControllerFSM.Default", -1);
        }

    }

    protected static class PhoneControllerFSM_Default
        extends PhoneControllerState
    {
    //-----------------------------------------------------------
    // Member methods.
    //

        protected PhoneControllerFSM_Default(String name, int id)
        {
            super (name, id);
        }

    //-----------------------------------------------------------
    // Inner classse.
    //


        private static final class PhoneControllerFSM_Wait_for_Call
            extends PhoneControllerFSM_Default
        {
        //-------------------------------------------------------
        // Member methods.
        //

            private PhoneControllerFSM_Wait_for_Call(String name, int id)
            {
                super (name, id);
            }

            protected void haveCall(PhoneControllerContext context, String number)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Waiting_for_Answer);
                (context.getState()).Entry(context);
                return;
            }

            protected void makeCall(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Dialing);
                (context.getState()).Entry(context);
                return;
            }

        //-------------------------------------------------------
        // Member data.
        //
        }

        private static final class PhoneControllerFSM_Dialing
            extends PhoneControllerFSM_Default
        {
        //-------------------------------------------------------
        // Member methods.
        //

            private PhoneControllerFSM_Dialing(String name, int id)
            {
                super (name, id);
            }

            protected void cancelCall(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Wait_for_Call);
                (context.getState()).Entry(context);
                return;
            }

            protected void dial(PhoneControllerContext context, String number)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Calling);
                (context.getState()).Entry(context);
                return;
            }

        //-------------------------------------------------------
        // Member data.
        //
        }

        private static final class PhoneControllerFSM_Calling
            extends PhoneControllerFSM_Default
        {
        //-------------------------------------------------------
        // Member methods.
        //

            private PhoneControllerFSM_Calling(String name, int id)
            {
                super (name, id);
            }

            protected void callFailure(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Dialing);
                (context.getState()).Entry(context);
                return;
            }

            protected void callSuccess(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Answering);
                (context.getState()).Entry(context);
                return;
            }

        //-------------------------------------------------------
        // Member data.
        //
        }

        private static final class PhoneControllerFSM_Answering
            extends PhoneControllerFSM_Default
        {
        //-------------------------------------------------------
        // Member methods.
        //

            private PhoneControllerFSM_Answering(String name, int id)
            {
                super (name, id);
            }

            protected void Entry(PhoneControllerContext context)
            {
                PhoneController ctxt = context.getOwner();

                ctxt.logStartTime();
                return;
            }

            protected void Exit(PhoneControllerContext context)
            {
                PhoneController ctxt = context.getOwner();

                ctxt.logEndTime();
                return;
            }

            protected void callFinished(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Wait_for_Call);
                (context.getState()).Entry(context);
                return;
            }

        //-------------------------------------------------------
        // Member data.
        //
        }

        private static final class PhoneControllerFSM_Waiting_for_Answer
            extends PhoneControllerFSM_Default
        {
        //-------------------------------------------------------
        // Member methods.
        //

            private PhoneControllerFSM_Waiting_for_Answer(String name, int id)
            {
                super (name, id);
            }

            protected void answer(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Answering);
                (context.getState()).Entry(context);
                return;
            }

            protected void noAnswer(PhoneControllerContext context)
            {


                (context.getState()).Exit(context);
                context.setState(PhoneControllerFSM.Wait_for_Call);
                (context.getState()).Entry(context);
                return;
            }

        //-------------------------------------------------------
        // Member data.
        //
        }

    //-----------------------------------------------------------
    // Member data.
    //
    }
}
