package sample.ui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;

import sample.control.PhoneController;
import sample.control.State;
import sample.control.StateChangeListener;

public class PhoneControllerFrame extends JFrame {
	private PhoneController _controller;
	
	private Display _display;
	private DialButton _dialButton;
	private Keypad _keypad;
	private CancelButton _cancelButton;
	private AnswerButton _answerButton;
	private MakeCallButton _makeCallButton;

	public PhoneControllerFrame(PhoneController aController) {
		super("Phone Controller Demo");
		_controller = aController;
		_controller.getContext().addStateChangeListener( new StateChangeListener() {
			public void stateChanged(State aOldState, State aNewState) {
				PhoneControllerFrame.this.stateChanged(aOldState, aNewState);
			}
		});
		initUI();
		stateChanged(null, _controller.getContext().getState());
	}

	public void stateChanged(sample.control.State aOldState, sample.control.State aNewState) {
		String oldStateName = aOldState == null ? "" : aOldState.getName();
		String newStateName = aNewState.getName();
		System.out.println(oldStateName +" -> " + newStateName);
		
		if (newStateName.equals("PhoneControllerFSM.Waiting_for_Call")) {
			// waiting for call
			_keypad.setEnabled(false);
			_answerButton.setEnabled(false);
			_makeCallButton.setEnabled(true);
			_cancelButton.setEnabled(false);
			_dialButton.setEnabled(false);
			
			if ("PhoneControllerFSM.Answering".equals(oldStateName))
				_display.setText("Total duration :"	+ _controller.getAnsweringDuration());
			else
				_display.setText("Waiting for call..");
		} else if ("PhoneControllerFSM.Dialing".equals(newStateName)) {
			// dialing
			_keypad.setEnabled(true);
			_answerButton.setEnabled(false);
			_makeCallButton.setEnabled(false);
			_cancelButton.setEnabled(true);
			_dialButton.setEnabled(true);
			_display.setText("");
		} else if ("PhoneControllerFSM.Calling".equals(newStateName)) {
			// calling
			_keypad.setEnabled(false);
			_answerButton.setEnabled(false);
			_makeCallButton.setEnabled(false);
			_cancelButton.setEnabled(false);
			_dialButton.setEnabled(false);
			_display.setText("Callling " + _controller.getDialingNumber()+ "...");

			boolean success = JOptionPane.showConfirmDialog(this, "Calling["+ _controller.getDialingNumber() + "] success?", "Calling...", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
			if (success)
				_controller.callSuccess();
			else
				_controller.callFailure();
		} else if ("PhoneControllerFSM.Answering".equals(newStateName)) {
			// answer
			_keypad.setEnabled(false);
			_answerButton.setEnabled(false);
			_makeCallButton.setEnabled(false);
			_cancelButton.setEnabled(true);
			_dialButton.setEnabled(false);
			_display.setText("Answering...");
		} else if ("PhoneControllerFSM.Waiting_for_Answer".equals(newStateName)) {
			_keypad.setEnabled(false);
			_answerButton.setEnabled(true);
			_makeCallButton.setEnabled(false);
			_cancelButton.setEnabled(true);
			_dialButton.setEnabled(false);
			_display.setText("Incoming Call : "	+ _controller.getIncomingCallNumber());
		} else {
			System.out.println("No this State: " + newStateName);
		}
	}

	private void initUI() {
		Container container = getContentPane();
		container.setLayout(new BorderLayout(5, 5));
		
		_display = new Display();
		_keypad = new Keypad(_display);
		
		JPanel bottomPanel = new JPanel(new GridLayout(2, 1));
		
		JPanel controlPane = new JPanel();
		_answerButton = new AnswerButton(_controller);
		_makeCallButton = new MakeCallButton(_controller);
		_dialButton = new DialButton(_controller, _display);
		_cancelButton = new CancelButton(_controller);
		
		controlPane.add(_answerButton);
		controlPane.add(_makeCallButton);
		
		JPanel dialPane = new JPanel();
		dialPane.add(_dialButton);
		dialPane.add(_cancelButton);
		
		bottomPanel.add(controlPane);
		bottomPanel.add(dialPane);
		
		container.add(_display, BorderLayout.NORTH);
		container.add(_keypad);
		container.add(bottomPanel, BorderLayout.SOUTH);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	
	}

	public static void main(String[] aArgs) throws Exception{
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

		PhoneController controller = new PhoneController();
		PhoneControllerFrame frame = new PhoneControllerFrame(controller);
		PhoneControllerTester tester = new PhoneControllerTester(controller);
		PhoneControllerLogger logger = new PhoneControllerLogger(controller);

		frame.pack();
		tester.pack();
		logger.setSize(300, 150);

		frame.setLocation(0, 0);
		tester.setLocation(frame.getSize().width, 0);
		logger.setLocation(frame.getSize().width, tester.getSize().height);

		frame.setVisible(true);
		tester.setVisible(true);
		logger.setVisible(true);
	}
}